package org.dasein.cloud.google;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.services.dc.DataCenterServices;
import org.dasein.cloud.services.storage.StorageServices;

public class GoogleAppEngine extends AbstractCloud {
    static public final String P_CFAUTH            = "Authorization";
    static public final String P_DATE              = "x-goog-date";
    
    static public String encode(String value, boolean encodePath) throws InternalException {
        String encoded = null;
        
        try {
            encoded = URLEncoder.encode(value, "utf-8").replace("+", "%20").replace("*", "%2A").replace("%7E","~");
            if( encodePath ) {
                encoded = encoded.replace("%2F", "/");
            }
        } 
        catch( UnsupportedEncodingException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        }
        return encoded;     
    }
    
    public GoogleAppEngine() { }
    
    @Override
    public DataCenterServices getDataCenterServices() {
        return new Dc(this);
    }
    
    @Override
    public StorageServices getStorageServices() {
        return new Storage(this);
    }
    
    @Override
    public String getCloudName() {
        return "Google AppEngine";
    }

    @Override
    public String getProviderName() {
        return "Google";
    }

    private String sign(byte[] key, String authString, String algorithm) throws InternalException {
        try {
            Mac mac = Mac.getInstance(algorithm);
            
            mac.init(new SecretKeySpec(key, algorithm));
            return new String(Base64.encodeBase64(mac.doFinal(authString.getBytes("utf-8"))));
        } 
        catch( NoSuchAlgorithmException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( InvalidKeyException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( IllegalStateException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( UnsupportedEncodingException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        }
    }
    
    String signForStorage(String accessKey, byte[] secretKey, String action, String hash, String contentType, Map<String,String> headers, String bucket, String object) throws InternalException {
        StringBuilder toSign = new StringBuilder();
        
        toSign.append(action);
        toSign.append("\n");
        if( hash != null ) {
            toSign.append(hash);
        }
        toSign.append("\n");
        if( contentType != null ) {
            toSign.append(contentType);
        }
        toSign.append("\n\n");
        ArrayList<String> keys = new ArrayList<String>();
        keys.addAll(headers.keySet());
        Collections.sort(keys);
        for( String hkey : keys ) {
            if( hkey.startsWith("x-amz") || hkey.startsWith("x-goog")) {
                String val = headers.get(hkey);
                
                if( val != null ) {
                    toSign.append(hkey);
                    toSign.append(":");
                    toSign.append(headers.get(hkey).trim());
                    toSign.append("\n");
                }
            }
        }
        toSign.append("/");
        if( bucket != null ) {
            toSign.append(bucket.toLowerCase());
            toSign.append("/");
        }
        if( object != null ) {
            toSign.append(object.toLowerCase());
        }
        String signature = sign(secretKey, toSign.toString(), "HmacSHA1");
        
        return ("GOOG1" + " " + accessKey + ":" + signature);
    }
}
